import { RandomState } from './random-state';
import { DecisionState } from './decision-state';
import { ActionTimeout, Answer, GameStart, OperationTimeout, Random, RandomnessReady, Shutdown, WaitingTimeout, } from './events';
import { SettleAdd, SettleEject, SettleSub } from './effect';
const OPERATION_TIMEOUT = 15000n;
export class GameContext {
    constructor(gameAccountOrContext) {
        if (gameAccountOrContext instanceof GameContext) {
            const context = gameAccountOrContext;
            this.gameAddr = context.gameAddr;
            this.accessVersion = context.accessVersion;
            this.settleVersion = context.settleVersion;
            this.transactorAddr = context.transactorAddr;
            this.status = context.status;
            this.players = context.players.map(p => Object.assign({}, p));
            this.servers = context.servers.map(s => Object.assign({}, s));
            this.dispatch = context.dispatch;
            this.handlerState = new Uint8Array(context.handlerState);
            this.timestamp = context.timestamp;
            this.allowExit = context.allowExit;
            this.randomStates = context.randomStates;
            this.decisionStates = context.decisionStates;
            this.settles = context.settles;
        }
        else {
            const gameAccount = gameAccountOrContext;
            const transactorAddr = gameAccount.transactorAddr;
            if (transactorAddr === undefined) {
                throw new Error('Game not served');
            }
            const players = gameAccount.players.map(p => ({
                addr: p.addr,
                balance: p.balance,
                position: p.position,
                status: {
                    kind: 'pending',
                    accessVersion: p.accessVersion,
                },
            }));
            const servers = gameAccount.servers.map(s => ({
                addr: s.addr,
                endpoint: s.endpoint,
                status: {
                    kind: 'pending',
                    accessVersion: s.accessVersion,
                },
            }));
            this.gameAddr = gameAccount.addr;
            this.transactorAddr = transactorAddr;
            this.accessVersion = gameAccount.accessVersion;
            this.settleVersion = gameAccount.settleVersion;
            this.status = 'uninit';
            this.dispatch = undefined;
            this.players = players;
            this.servers = servers;
            this.timestamp = 0n;
            this.allowExit = false;
            this.randomStates = [];
            this.decisionStates = [];
            this.settles = undefined;
            this.handlerState = Uint8Array.of();
        }
    }
    getServerByAddress(addr) {
        return this.servers.find(s => s.addr === addr);
    }
    getPlayerByAddress(addr) {
        return this.players.find(p => p.addr === addr);
    }
    dispatchEvent(event, timeout) {
        this.dispatch = {
            event,
            timeout: this.timestamp + timeout,
        };
    }
    dispatchEventInstantly(event) {
        this.dispatchEvent(event, 0n);
    }
    waitTimeout(timeout) {
        this.dispatch = {
            event: new WaitingTimeout({}),
            timeout: this.timestamp + timeout,
        };
    }
    actionTimeout(playerAddr, timeout) {
        this.dispatch = {
            event: new ActionTimeout({ playerAddr }),
            timeout: this.timestamp + timeout,
        };
    }
    genStartGameEvent() {
        return new GameStart({ accessVersion: this.accessVersion });
    }
    startGame() {
        this.randomStates = [];
        this.decisionStates = [];
        this.dispatch = {
            event: this.genStartGameEvent(),
            timeout: 0n,
        };
    }
    shutdownGame() {
        this.dispatch = {
            event: new Shutdown({}),
            timeout: 0n,
        };
    }
    getRandomState(randomId) {
        if (randomId <= 0) {
            throw new Error('Invalid random id: ' + randomId);
        }
        const st = this.randomStates[randomId - 1];
        if (st === undefined) {
            throw new Error('Invalid random id: ' + randomId);
        }
        return st;
    }
    getDecisionState(decisionId) {
        if (decisionId <= 0) {
            throw new Error('Invalid decision id: ' + decisionId);
        }
        const st = this.decisionStates[decisionId - 1];
        if (st === undefined) {
            throw new Error('Invalid decision id: ' + decisionId);
        }
        return st;
    }
    assign(randomId, playerAddr, indexes) {
        const st = this.getRandomState(randomId);
        st.assign(playerAddr, indexes);
    }
    reveal(randomId, indexes) {
        const st = this.getRandomState(randomId);
        st.reveal(indexes);
    }
    isRandomReady(randomId) {
        const k = this.getRandomState(randomId).status.kind;
        return k === 'ready' || k === 'waiting-secrets';
    }
    isAllRandomReady() {
        for (const st of this.randomStates) {
            const k = st.status.kind;
            if (k !== 'ready' && k !== 'waiting-secrets') {
                return false;
            }
        }
        return true;
    }
    isSecretsReady() {
        return this.randomStates.every(st => st.status.kind === 'ready');
    }
    setPlayerStatus(addr, status) {
        let p = this.players.find(p => p.addr === addr);
        if (p === undefined) {
            throw new Error('Invalid player address');
        }
        p.status = status;
    }
    addPlayer(player) {
        const exist = this.players.find(p => p.addr === player.addr || p.position === player.position);
        if (exist === undefined) {
            this.players.push({
                addr: player.addr,
                balance: player.balance,
                status: { kind: 'ready' },
                position: player.position,
            });
        }
        else {
            if (exist.position === player.position) {
                throw new Error('Position occupied');
            }
            else {
                throw new Error('Player already joined');
            }
        }
    }
    addServer(server) {
        const exist = this.players.find(s => s.addr === server.addr);
        if (exist === undefined) {
            this.servers.push({
                addr: server.addr,
                status: { kind: 'ready' },
                endpoint: server.endpoint,
            });
        }
        else {
            throw new Error('Server already joined');
        }
    }
    setAccessVersion(accessVersion) {
        this.accessVersion = accessVersion;
    }
    setAllowExit(allowExit) {
        this.allowExit = allowExit;
    }
    removePlayer(addr) {
        if (this.allowExit) {
            const origLen = this.players.length;
            this.players = this.players.filter(p => p.addr !== addr);
            if (this.players.length === origLen) {
                throw new Error('Player not in game');
            }
        }
        else {
            throw new Error("Can't leave");
        }
    }
    initRandomState(spec) {
        const randomId = this.randomStates.length + 1;
        const owners = this.servers.filter(s => s.status.kind === 'ready').map(s => s.addr);
        const randomState = new RandomState(randomId, spec, owners);
        this.randomStates.push(randomState);
        return randomId;
    }
    addSharedSecrets(_addr, shares) {
        for (const share of shares) {
            if (share instanceof Random) {
                const { randomId, toAddr, fromAddr, index, secret } = share;
                this.getRandomState(randomId).addSecret(fromAddr, toAddr, index, secret);
            }
            else if (share instanceof Answer) {
                const { fromAddr, decisionId, secret } = share;
                this.getDecisionState(decisionId).addSecret(fromAddr, secret);
            }
        }
    }
    randomizeAndMask(addr, randomId, ciphertexts) {
        let st = this.getRandomState(randomId);
        st.mask(addr, ciphertexts);
        this.dispatchRandomizationTimeout(randomId);
    }
    lock(addr, randomId, ciphertextsAndTests) {
        let st = this.getRandomState(randomId);
        st.lock(addr, ciphertextsAndTests);
        this.dispatchRandomizationTimeout(randomId);
    }
    dispatchRandomizationTimeout(randomId) {
        const noDispatch = this.dispatch === undefined;
        let st = this.getRandomState(randomId);
        const statusKind = st.status.kind;
        if (statusKind === 'ready') {
            this.dispatchEventInstantly(new RandomnessReady({ randomId }));
        }
        else if (statusKind === 'locking' || statusKind === 'masking') {
            const addr = st.status.addr;
            if (noDispatch) {
                this.dispatchEvent(new OperationTimeout({ addrs: [addr] }), OPERATION_TIMEOUT);
            }
        }
        else if (statusKind === 'waiting-secrets') {
            if (noDispatch) {
                const addrs = st.listOperatingAddrs();
                this.dispatchEvent(new OperationTimeout({ addrs }), OPERATION_TIMEOUT);
            }
        }
    }
    settle(settles) {
        this.settles = settles;
    }
    bumpSettleVersion() {
        this.settleVersion += 1n;
    }
    applyAndTakeSettles() {
        if (this.settles === undefined) {
            return undefined;
        }
        let settles = this.settles;
        this.settles = undefined;
        settles = settles.sort((s1, s2) => s1.compare(s2));
        for (const s of settles) {
            if (s.op instanceof SettleAdd) {
                let p = this.getPlayerByAddress(s.addr);
                if (p === undefined) {
                    throw new Error('Invalid settle player address');
                }
                p.balance += s.op.amount;
            }
            else if (s.op instanceof SettleSub) {
                let p = this.getPlayerByAddress(s.addr);
                if (p === undefined) {
                    throw new Error('Invalid settle player address');
                }
                p.balance -= s.op.amount;
            }
            else if (s.op instanceof SettleEject) {
                this.players = this.players.filter(p => p.addr !== s.addr);
            }
        }
        this.bumpSettleVersion();
        return settles;
    }
    addSettle(settle) {
        if (this.settles === undefined) {
            this.settles = [settle];
        }
        else {
            this.settles.push(settle);
        }
    }
    addRevealedRandom(randomId, revealed) {
        const st = this.getRandomState(randomId);
        st.addRevealed(revealed);
    }
    addRevealedAnswer(decisionId, revealed) {
        const st = this.getDecisionState(decisionId);
        st.addReleased(revealed);
    }
    ask(owner) {
        const id = this.decisionStates.length + 1;
        const st = new DecisionState(id, owner);
        this.decisionStates.push(st);
        return id;
    }
    answerDecision(id, owner, ciphertext, digest) {
        const st = this.getDecisionState(id);
        st.setAnswer(owner, ciphertext, digest);
    }
    getRevealed(randomId) {
        let st = this.getRandomState(randomId);
        return st.revealed;
    }
    applyEffect(effect) {
        if (effect.startGame) {
            this.startGame();
        }
        else if (effect.stopGame) {
            this.shutdownGame();
        }
        else if (effect.actionTimeout !== undefined) {
            this.actionTimeout(effect.actionTimeout.playerAddr, effect.actionTimeout.timeout);
        }
        else if (effect.waitTimeout !== undefined) {
            this.waitTimeout(effect.waitTimeout);
        }
        else if (effect.cancelDispatch) {
            this.dispatch = undefined;
        }
        for (const assign of effect.assigns) {
            this.assign(assign.randomId, assign.playerAddr, assign.indexes);
        }
        for (const reveal of effect.reveals) {
            this.reveal(reveal.randomId, reveal.indexes);
        }
        for (const ask of effect.asks) {
            this.ask(ask.playerAddr);
        }
        for (const spec of effect.initRandomStates) {
            this.initRandomState(spec);
        }
        if (effect.settles.length > 0) {
            this.settle(effect.settles);
        }
        if (effect.handlerState !== undefined) {
            this.handlerState = effect.handlerState;
        }
    }
    setNodeReady(accessVersion) {
        for (const s of this.servers) {
            if (s.status.kind === 'pending') {
                if (s.status.accessVersion < accessVersion) {
                    s.status = { kind: 'ready' };
                }
            }
        }
        for (const p of this.players) {
            if (p.status.kind === 'pending') {
                if (p.status.accessVersion < accessVersion) {
                    p.status = { kind: 'ready' };
                }
            }
        }
    }
    applyCheckpoint(accessVersion, settleVersion) {
        if (this.settleVersion !== settleVersion) {
            throw new Error('Invalid checkpoint');
        }
        this.players = this.players.filter(p => {
            if (p.status.kind === 'pending') {
                return p.status.accessVersion <= accessVersion;
            }
            else {
                return true;
            }
        });
        this.servers = this.servers.filter(s => {
            if (s.status.kind === 'pending') {
                return s.status.accessVersion <= accessVersion;
            }
            else {
                return true;
            }
        });
        this.accessVersion = accessVersion;
    }
}
//# sourceMappingURL=game-context.js.map