import { Secret, Ciphertext } from './types';
export declare const aesContentIv: Uint8Array;
export declare const chacha20Nonce: Uint8Array;
export declare const aesDigestIv: Uint8Array;
export interface INodePrivateKey {
    rsa: CryptoKeyPair;
    ec: CryptoKeyPair;
}
export interface INodePublicKey {
    rsa: CryptoKey;
    ec: CryptoKey;
}
export interface IPublicKeyRaws {
    rsa: string;
    ec: string;
}
export declare class PublicKeyRaws {
    rsa: string;
    ec: string;
    constructor(fields: IPublicKeyRaws);
}
export declare function exportRsaPublicKey(publicKey: CryptoKey): Promise<string>;
export declare function exportEcPublicKey(publicKey: CryptoKey): Promise<string>;
export declare function exportAes(key: CryptoKey): Promise<Uint8Array>;
export declare function exportRsa(keypair: CryptoKeyPair): Promise<[string, string]>;
export declare function exportEc(keypair: CryptoKeyPair): Promise<[string, string]>;
export declare function encryptRsa(publicKey: CryptoKey, plaintext: Uint8Array): Promise<Uint8Array>;
export declare function decryptRsa(privateKey: CryptoKey, ciphertext: Uint8Array): Promise<Uint8Array>;
export declare function signEc(privateKey: CryptoKey, message: Uint8Array): Promise<Uint8Array>;
export declare function verifyEc(publicKey: CryptoKey, signature: Uint8Array, message: Uint8Array): Promise<boolean>;
export declare function encryptChacha20(key: Uint8Array, text: Uint8Array, nonce: Uint8Array): Uint8Array;
export declare function decryptChacha20(key: Uint8Array, text: Uint8Array, nonce: Uint8Array): Uint8Array;
export declare function encryptAes(key: CryptoKey, text: Uint8Array, iv: Uint8Array): Promise<Uint8Array>;
export declare function decryptAes(key: CryptoKey, text: Ciphertext, iv: Uint8Array): Promise<Uint8Array>;
export declare function importAes(rawKey: Uint8Array): Promise<CryptoKey>;
export declare function importRsa([privateKeyStr, publicKeyStr]: [string, string]): Promise<CryptoKeyPair>;
export declare function importEc([privateKeyStr, publicKeyStr]: [string, string]): Promise<CryptoKeyPair>;
export declare function importRsaPublicKey(publicKeyStr: string): Promise<CryptoKey>;
export declare function importEcPublicKey(publicKeyStr: string): Promise<CryptoKey>;
export declare function generateEcKeypair(): Promise<CryptoKeyPair>;
export declare function generateRsaKeypair(): Promise<CryptoKeyPair>;
export declare function generateChacha20(): Uint8Array;
export declare function generateAes(): Promise<CryptoKey>;
export interface ISignature {
    signer: string;
    timestamp: bigint;
    signature: Uint8Array;
}
export declare class Signature {
    signer: string;
    timestamp: bigint;
    signature: Uint8Array;
    constructor(fields: ISignature);
}
/**
 * Encryptor
 * Use RSA and ChaCha20(AES-CTR) for random secrets encryption.
 */
export interface IEncryptor {
    addPublicKey(addr: string, pubkeys: IPublicKeyRaws): Promise<void>;
    exportPublicKey(addr?: string): Promise<IPublicKeyRaws>;
    decryptRsa(text: Uint8Array): Promise<Uint8Array>;
    decryptAes(secret: Secret, text: Ciphertext): Promise<Ciphertext>;
    decryptAesMulti(secrets: Secret[], text: Ciphertext): Promise<Ciphertext>;
    encryptChacha20(secret: Secret, text: Ciphertext): Ciphertext;
    decryptChacha20(secret: Secret, text: Ciphertext): Ciphertext;
    sign(message: Uint8Array, signer: string): Promise<Signature>;
    verify(message: Uint8Array, signature: Signature): Promise<boolean>;
    decryptWithSecrets(ciphertextMap: Map<number, Ciphertext>, secretMap: Map<number, Secret[]>, validOptions: string[]): Promise<Map<number, string>>;
}
export declare class Encryptor implements IEncryptor {
    #private;
    constructor(priv: INodePrivateKey);
    decryptRsa(text: Uint8Array): Promise<Uint8Array>;
    decryptAes(secret: Secret, text: Ciphertext): Promise<Ciphertext>;
    decryptAesMulti(secrets: Secret[], text: Ciphertext): Promise<Ciphertext>;
    encryptChacha20(secret: Secret, text: Ciphertext): Ciphertext;
    decryptChacha20(secret: Secret, text: Ciphertext): Ciphertext;
    decryptChacha20Multi(secrets: Secret[], text: Ciphertext): Ciphertext;
    signRaw(message: Uint8Array): Promise<Uint8Array>;
    makeSignMessage(message: Uint8Array, timestamp: bigint): Uint8Array;
    sign(message: Uint8Array, signer: string): Promise<Signature>;
    verify(message: Uint8Array, signature: Signature): Promise<boolean>;
    static create(): Promise<Encryptor>;
    exportRsaKeys(): Promise<[string, string]>;
    exportEcKeys(): Promise<[string, string]>;
    addPublicKey(addr: string, { rsa, ec }: IPublicKeyRaws): Promise<void>;
    exportPublicKey(addr?: string): Promise<IPublicKeyRaws>;
    decryptWithSecrets(ciphertextMap: Map<number, Ciphertext>, secretMap: Map<number, Secret[]>, validOptions: string[]): Promise<Map<number, string>>;
}
//# sourceMappingURL=encryptor.d.ts.map